/** @jsx React.DOM */
// Tooltip component and mixin
// The consumer of <Tooltip> must use the mixin to get event handler
// functionality set up correctly.

define('common_controls',
      ['react', 'common_util', 'taskqueue', 'jquery'],
      function(React, Util, TaskQueue, jQuery) {

  var Tooltip = React.createClass({displayName: 'Tooltip',
    render: function() {
      var style = {
        'display': this.props.show ? '' : 'none',
        'top': this.props.y - 50,
        'left': this.props.x - 150,
      };
      return (
        React.DOM.div( {style:style, className:"canvas-tooltip"}, 
          this.props.data
        )
      );
    }
  });

  var Modal = React.createClass({displayName: 'Modal',
    doNothing: function(evt) {evt.stopPropagation();},
    render: function() {
      return (
        React.DOM.div(null, 
          React.DOM.div( {onClick:this.props.closeModal, className:'modal fade' + (this.props.open ? ' in' : ''), style:{'display': (this.props.open ? 'block' : 'none')}}, 
            React.DOM.div( {onClick:this.doNothing, className:"modal-dialog"}, 
              React.DOM.div( {className:"modal-content"}, 
                React.DOM.div( {className:"modal-header"}, 
                  React.DOM.button( {type:"button", className:"close", onClick:this.props.closeModal}, "×"),
                  React.DOM.h4( {className:"modal-title"}, this.props.title)
                ),
                React.DOM.div( {className:"modal-body"}, 
                  this.props.body
                )
              )
            )
          ),
          React.DOM.div( {className:'modal-backdrop fade ' + (this.props.open ? 'in' : 'hidden')} )
        )
      );
    }
  });

  var SizeToWindow = {
    setWindowDimensions: function() {
      this.setState({
        'windowHeight': $(window).height(),
        'windowWidth': $(window).width()
      });
    },
    componentWillMount: function() {
      $(window).bind('resize', this.setWindowDimensions);
      this.setWindowDimensions();
    },
    componentWillUnmount: function() {
      $(window).unbind('resize', this.setWindowDimensions);
    },
  };

  return {
    Slider: React.createClass({displayName: 'Slider',
      render: function() {
        Util.assert(this.props.value >= this.props.min);
        Util.assert(this.props.value <= this.props.max);
        return (
          React.DOM.div( {className:"canvas-slider"}, 
            React.DOM.div( {className:"canvas-slider-label"}, 
              this.props.label
            ),
            React.DOM.div( {className:"canvas-slider-control"}, 
              this.props.min,
              this.transferPropsTo(
                React.DOM.input( {type:"range", name:"canvas-slider-range"} )
              ),
              this.props.max
            ),
            React.DOM.div( {className:"canvas-slider-box"}, 
              this.transferPropsTo(
                React.DOM.input( {type:"number", name:"canvas-slider-number"} )
              )
            )
          )
        );
      }
    }),
    Tooltip: {
      // Mixin this to consume <Tooltip>
      getInitialState: function() {
        return {
          'tooltip': {
            'show': false,
            'x': 0,
            'y': 0,
            'data': null,
            'timeoutFn': null
          }
        };
      },
      tooltipMouseOver: function(data, evt) {
        this.setState({
          'tooltip': {
            'show': false,
            'x': evt.clientX,
            'y': evt.clientY,
            'data': data,
            'timeoutFn': setTimeout(function() {
              this.setState({
                'tooltip': Util.merge(this.state.tooltip, {
                  'show': true
                })
              });
            }.bind(this), 400)
          }
        });
      },
      tooltipMouseOut: function(evt) {
        clearTimeout(this.state.tooltip.timeoutFn);
        this.setState({
          'tooltip': {
            'show': false,
            'timeoutFn': null
          }
        });
      },
      getTooltip: function() {
        return (
          Tooltip(
            {data:this.state.tooltip.data,
            x:this.state.tooltip.x,
            y:this.state.tooltip.y - 20,
            show:this.state.tooltip.show}
          )
        );
      }
    },
    Modal: {
      // Mixin this to consume <Modal>
      getInitialState: function() {
        return {'modal': {'open': false}};
      },
      modalOpen: function(title, body) {
        this.setState({'modal': {
          'open': true,
          'title': title,
          'body': body
        }});
      },
      modalClose: function() {
        this.setState({'modal': {'open': false}});
      },
      getModal: function() {
        return (
          Modal(
            {open:this.state.modal.open,
            closeModal:this.modalClose,
            title:this.state.modal.title,
            body:this.state.modal.body}
          )
        );
      }
    },
    SizeToWindow: SizeToWindow,
    Tabs: React.createClass({displayName: 'Tabs',
      mixins: [SizeToWindow],
      getInitialState: function() {
        // keep track of previously selected tabs so they can
        // maintain state as the user switches between tabs.
        // but don't render them unless they've been selected before.
        return {
          selectedTab: this.props.tabs[0],
          previouslySelectedTabs: {}
        };
      },
      componentWillReceiveProps: function(nextProps) {
        if (nextProps.tabs.indexOf(this.state.selectedTab) === -1) {
          this.setState({selectedTab: nextProps.tabs[0]});
        }
      },
      selectTab: function(tab, evt) {
        var prev = {};
        prev[this.state.selectedTab] = true;
        this.setState({
          selectedTab: tab,
          previouslySelectedTabs: $.extend(true, {}, this.state.previouslySelectedTabs, prev)
        });
      },
      render: function() {
        return (
          React.DOM.div(null, 
            React.DOM.ul( {className:"nav nav-tabs", role:"tablist"}, 
              this.props.tabs.map(function(tab, idx) {
                return (
                  React.DOM.li( {key:idx, className:tab === this.state.selectedTab ? 'active' : ''}, 
                    React.DOM.a( {href:"javascript:", onClick:this.selectTab.bind(this, tab)}, 
                      tab
                    )
                  )
                );
              }.bind(this))
            ),
            React.DOM.div( {className:"tab-content", style:{
              width: this.state.windowWidth - (190 + 50),
              height: this.state.windowHeight - (74 + 50),
              overflow: 'auto'
            }}, 
              this.props.tabContents.map(function(row, idx) {
                var tag = row[0];
                var props = row[1];
                var classNames = row[2];
                var tab = this.props.tabs[idx];
                if (tab === this.state.selectedTab) {
                  return (
                    React.DOM.div( {key:idx, className:'tab-pane active ' + classNames}, 
                      tag(props)
                    )
                  );
                } else if (tab in this.state.previouslySelectedTabs) {
                  return (
                    React.DOM.div( {key:idx, className:'tab-pane ' + classNames}, 
                      tag(props)
                    )
                  );
                } else {
                  return null;
                }
              }.bind(this))
            )
          )
        );
      }
    }),
    AsyncState: {
      /*
       * Mixin AsyncState to a React component and provide an updateAsyncState
       * method on that component in order to get the behavior that on mount
       * and whenever the this.props.selected_variable changes, the state will
       * get re-initialized to default and then updateAsyncState will get called.
       */

      componentWillReceiveProps: function(nextProps) {
        // if the selected_variable changes, we need to
        // invalidate our cached head and sketch and
        // get new ones.
        if (!Util.deepEquals(this.props, nextProps)) {
          this.setState(this.getInitialState());
          this.updateAsyncState(nextProps);
        }
      },
      addTask: function(task) {
        // adds a task to the task queue,
        // and will cancel it if the component gets unmounted
        this.tasks.push(task);
        TaskQueue.add(task);
      },
      componentDidMount: function() {
        // get initial async state
        this.tasks = [];
        this.updateAsyncState(this.props);
      },
      componentWillUnmount: function() {
        // cancel pending tasks for this component
        this.tasks.forEach(function(task) {
          task.cancel();
        });
      }
    },
    SVG: {
      /* SVG elements that are missing from React 0.10.0 */
      ellipse: React.createClass({displayName: 'ellipse',
        render: function() {
          // implement ellipse in terms of path
          var d = "M " +
                  (this.props.cx - this.props.rx) +
                  " " +
                  this.props.cy +
                  " a " +
                  this.props.rx +
                  " " +
                  this.props.ry +
                  " 0 1 0 " +
                  (this.props.rx * 2) +
                  " 0 a " +
                  this.props.rx +
                  " " +
                  this.props.ry +
                  " 0 1 0 " +
                  (-(this.props.rx * 2)) +
                  " 0";
          return this.transferPropsTo(
            React.DOM.path( {d:d} )
          );
        }
      })
    }
  };
});
